var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { asMaybe } from 'cleaners';
import { makeReactNativeDisklet } from 'disklet';
import * as React from 'react';
import { NativeModules, Platform } from 'react-native';
import { checkNotifications, openSettings, requestNotifications, RESULTS } from 'react-native-permissions';
import { sprintf } from 'sprintf-js';
import s from '../common/locales/strings';
import { RequestPermissionsModal } from '../components/modals/RequestPermissionsModal';
import { SecurityAlertsModal } from '../components/modals/SecurityAlertsModal';
import { Airship, showError } from '../components/services/AirshipInstance';
import { asNotificationPermissionsInfo } from '../types/ReduxTypes';
import { launchPasswordRecovery } from './LoginAction';
import { loadTouchState } from './TouchActions';
const { AbcCoreJsUi } = NativeModules;
const disklet = makeReactNativeDisklet();
const notificationPermissionsInfoFile = 'notificationsPermisions.json';
/**
 * Fires off all the things we need to do to get the login scene up & running.
 */
export const initializeLogin = (theme, branding) => (dispatch, getState, imports) => __awaiter(void 0, void 0, void 0, function* () {
    const { customPermissionsFunction } = imports;
    const touchPromise = dispatch(loadTouchState());
    dispatch(checkSecurityMessages()).catch(error => console.log(error));
    customPermissionsFunction
        ? customPermissionsFunction()
        : dispatch(checkAndRequestNotifications(theme, branding)).catch(error => console.log(error));
    yield touchPromise;
    const state = getState();
    // Loading is done, so send the user to the initial route:
    const biometryType = state.touch.type;
    const { startupUser } = state.previousUsers;
    const { recoveryKey } = imports;
    if (recoveryKey) {
        dispatch({ type: 'START_LANDING' });
        dispatch(launchPasswordRecovery(recoveryKey));
    }
    else if (startupUser == null) {
        dispatch({ type: 'START_LANDING' });
    }
    else if (startupUser.pinEnabled ||
        (startupUser.touchEnabled && biometryType !== false)) {
        dispatch({ type: 'START_PIN_LOGIN' });
    }
    else {
        dispatch({ type: 'START_PASSWORD_LOGIN' });
    }
});
const checkSecurityMessages = () => (dispatch, getState, imports) => __awaiter(void 0, void 0, void 0, function* () {
    const { context } = imports;
    const messages = yield context.fetchLoginMessages();
    const relevantMessages = {};
    for (const username of Object.keys(messages)) {
        const message = messages[username];
        // Skip users who haven't fully logged in:
        const info = context.localUsers.find(info => info.username === username);
        if (info == null || !info.keyLoginEnabled)
            continue;
        const { otpResetPending, pendingVouchers = [] } = message;
        if (otpResetPending || pendingVouchers.length > 0) {
            relevantMessages[username] = message;
        }
    }
    if (Object.keys(relevantMessages).length > 0) {
        Airship.show(bridge => (React.createElement(SecurityAlertsModal, { bridge: bridge, messages: relevantMessages, selectUser: username => dispatch({ type: 'AUTH_UPDATE_USERNAME', data: username }) })));
    }
});
//
// Logic for showing notification/app refresh request
//
// 0 = false, 1 = true
// nr = notifications and app refresh request message
// r = notifications request message
// r = app refresh request only message
//
// | Notif Enabled |  Notif Block | App Refresh Enabled || Message |
// |---------------|--------------|---------------------||---------|
// |       0       |       0      |           0         >>    nr   |
// |       0       |       0      |           1         >>    n    |
// |       0       |       1      |           0         >>    r    |
// |       0       |       1      |           1         >>         |
// |       1       |       0      |           0         >>    r    |
// |       1       |       0      |           1         >>         |
// |       1       |       1      |           0         >>    r    |
// |       1       |       1      |           1         >>         |
//
const isIos = Platform.OS === 'ios';
const logicMap = [
    [[], []],
    [[], []]
];
logicMap[0][0][0] = s.strings.notifications_and_refresh_permissions_branded;
logicMap[0][0][1] = s.strings.notifications_permissions_branded;
logicMap[0][1][0] = s.strings.refresh_permission_branded;
logicMap[0][1][1] = undefined;
logicMap[1][0][0] = s.strings.refresh_permission_branded;
logicMap[1][0][1] = undefined;
logicMap[1][1][0] = s.strings.refresh_permission_branded;
logicMap[1][1][1] = undefined;
const checkAndRequestNotifications = (theme, branding) => (dispatch, getState, imports) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b, _c;
    const { onNotificationPermit } = imports;
    const notificationPermission = yield checkNotifications();
    const notificationStatus = notificationPermission.status;
    const notifEnabled = notificationStatus !== RESULTS.BLOCKED &&
        notificationStatus !== RESULTS.DENIED
        ? 1
        : 0;
    const statusAppRefresh = isIos
        ? yield AbcCoreJsUi.backgroundAppRefreshStatus().catch((error) => console.log(error))
        : undefined;
    const refreshEnabled = statusAppRefresh !== RESULTS.BLOCKED ? 1 : 0;
    const notificationPermissionsInfoJson = yield disklet
        .getText(notificationPermissionsInfoFile)
        .catch(error => console.log(error));
    const notificationPermissionsInfo = asMaybe(asNotificationPermissionsInfo)(notificationPermissionsInfoJson);
    const isNotificationBlocked = notificationPermissionsInfo
        ? notificationPermissionsInfo.isNotificationBlocked
        : false;
    const isNotificationBlockedBit = isNotificationBlocked ? 1 : 0;
    const permissionMessage = logicMap[notifEnabled][isNotificationBlockedBit][refreshEnabled] != null
        ? sprintf((_a = logicMap[notifEnabled][isNotificationBlockedBit][refreshEnabled]) !== null && _a !== void 0 ? _a : '', branding.appName)
        : undefined;
    console.log(`checkAndRequestNotifications`);
    console.log(`notificationStatus:${notificationStatus} statusAppRefresh:${statusAppRefresh}`);
    console.log(`notifEnabled:${notifEnabled} notifBlocked:${isNotificationBlockedBit} refreshEnabled:${refreshEnabled}`);
    console.log(`permissionMessage:${permissionMessage}`);
    if (permissionMessage != null) {
        const choiceMap = {
            optInPriceChanges: !((_b = notificationPermissionsInfo === null || notificationPermissionsInfo === void 0 ? void 0 : notificationPermissionsInfo.notificationOptIns.ignorePriceChanges) !== null && _b !== void 0 ? _b : false),
            optInMarketing: !((_c = notificationPermissionsInfo === null || notificationPermissionsInfo === void 0 ? void 0 : notificationPermissionsInfo.notificationOptIns.ignoreMarketing) !== null && _c !== void 0 ? _c : false)
        };
        Airship.show(bridge => (React.createElement(RequestPermissionsModal, { bridge: bridge, message: permissionMessage, choices: choiceMap })))
            .then((choices) => __awaiter(void 0, void 0, void 0, function* () {
            console.log(`checkAndRequestNotifications result ${JSON.stringify(choices)}`);
            // Don't do anything if the user didn't make a choice (dismissed modal)
            if (choices == null)
                return;
            const notificationPermissionsInfo = {
                isNotificationBlocked: !(choices === null || choices === void 0 ? void 0 : choices.enable),
                notificationOptIns: {
                    ignoreMarketing: !choices.optInMarketing,
                    ignorePriceChanges: !choices.optInPriceChanges
                }
            };
            yield disklet.setText(notificationPermissionsInfoFile, JSON.stringify(notificationPermissionsInfo));
            if (choices.enable) {
                if (notificationStatus === RESULTS.DENIED) {
                    requestNotifications(isIos ? ['alert', 'badge', 'sound'] : []).catch(error => console.log(error));
                }
                if (notificationStatus === RESULTS.BLOCKED ||
                    statusAppRefresh === RESULTS.BLOCKED) {
                    openSettings().catch(error => console.log(error));
                }
            }
            if (onNotificationPermit != null)
                onNotificationPermit(notificationPermissionsInfo);
        }))
            .catch(showError);
    }
    else {
        if (onNotificationPermit != null && notificationPermissionsInfo != null) {
            onNotificationPermit(notificationPermissionsInfo);
        }
    }
});
