var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as React from 'react';
import { View } from 'react-native';
import { KeyboardAwareScrollView } from 'react-native-keyboard-aware-scroll-view';
import { cacheStyles } from 'react-native-patina';
import { sprintf } from 'sprintf-js';
import { checkUsernameForAvailabilty, validateUsername } from '../../../actions/CreateAccountActions';
import s from '../../../common/locales/strings';
import { useHandler } from '../../../hooks/useHandler';
import { connect } from '../../services/ReduxStore';
import { withTheme } from '../../services/ThemeContext';
import { EdgeText } from '../../themed/EdgeText';
import { MainButton } from '../../themed/MainButton';
import { OutlinedTextInput } from '../../themed/OutlinedTextInput';
import { ThemedScene } from '../../themed/ThemedScene';
const NewAccountUsernameSceneComponent = ({ theme, onBack, branding, username, usernameErrorMessage, checkUsernameForAvailabilty, validateUsername }) => {
    const styles = getStyles(theme);
    const handleNext = useHandler(() => __awaiter(void 0, void 0, void 0, function* () {
        if (usernameErrorMessage || !username) {
            return;
        }
        yield checkUsernameForAvailabilty(username);
    }));
    const error = usernameErrorMessage == null || usernameErrorMessage === ''
        ? undefined
        : usernameErrorMessage;
    return (React.createElement(ThemedScene, { onBack: onBack, title: s.strings.choose_title_username },
        React.createElement(View, { style: styles.content },
            React.createElement(KeyboardAwareScrollView, { contentContainerStyle: styles.mainScrollView, keyboardShouldPersistTaps: "handled" },
                React.createElement(EdgeText, { style: styles.description, numberOfLines: 2 }, sprintf(s.strings.username_desc, branding.appName || s.strings.app_name_default)),
                React.createElement(OutlinedTextInput, { autoCorrect: false, autoFocus: true, label: s.strings.username, onChangeText: validateUsername, onSubmitEditing: handleNext, returnKeyType: "go", marginRem: 1, value: username, clearIcon: true, error: error, searchIcon: false }),
                React.createElement(MainButton, { alignSelf: "center", label: s.strings.next_label, type: "secondary", marginRem: [1.5, 0.5], disabled: error != null, onPress: handleNext })))));
};
const getStyles = cacheStyles((theme) => ({
    content: {
        flex: 1,
        marginHorizontal: theme.rem(0.5),
        marginTop: theme.rem(1.5)
    },
    mainScrollView: {
        flex: 1,
        alignContent: 'flex-start'
    },
    description: {
        fontFamily: theme.fontFaceDefault,
        fontSize: theme.rem(0.875),
        marginBottom: theme.rem(1)
    }
}));
export const NewAccountUsernameScene = connect((state) => ({
    username: state.create.username || '',
    usernameErrorMessage: state.create.usernameErrorMessage
}), (dispatch) => ({
    onBack() {
        dispatch({ type: 'NEW_ACCOUNT_WELCOME' });
    },
    checkUsernameForAvailabilty(data) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield dispatch(checkUsernameForAvailabilty(data));
        });
    },
    validateUsername(username) {
        dispatch(validateUsername(username));
    }
}))(withTheme(NewAccountUsernameSceneComponent));
